/* mpivtopnm.c - convert an MPIV file to a set of pnm files */

#include <stdio.h>
#include <stdlib.h>
#include "pnmio.h"
#include "mpiv0.h"



#define		YES		-1
#define		NO		0


#define		UCHAR		unsigned char

#define MIN(x, y)  (((x) < (y)) ? (x) : (y))
#define MAX(x, y)  (((x) > (y)) ? (x) : (y))


void main( int argc, char **argv )
{
  FILE *fp;
  int fnumber;
  int xsize, ysize;
  MPIV0_FormatIndex format;
  UCHAR *r, *g, *b;
  char fname[1000];

  if ( argc != 3 ) {
    fprintf( stderr, "Usage: %s file.mpiv base\n", argv[0] );
    exit( YES );
  }

  /* read the mpiv header */
  if ( (fp = mpiv0_open( argv[1], &xsize, &ysize, &format )) == NULL ) {
    fprintf( stderr, "Error:  mpiv0_open failed for <%s>\n", argv[1] );
    exit( YES );
  }

  /* print nice message to console */
  switch( format ) {
  case RGB24 :
    fprintf( stdout, "Found RGB24 -> converting to ppm\n" );
    break;
  case RGB15 :
    fprintf( stdout, "Found RGB15 -> converting to ppm\n" );
    break;
  case RGB8 :
    fprintf( stdout, "Found RGB8 -> converting to ppm\n" );
    break;
  case BW8 :
    fprintf( stdout, "Found BW8 -> converting to pgm\n" );
    break;
  default :
      fprintf( stderr, "Error: illegal format encoutered\n" );
      exit( YES );
  }

  /* read the frames one at a time and save them to ppm or pgm files */
  for( fnumber = 0; mpiv0_read( fp, xsize, ysize, format, &r, &g, &b ) == 1;
      fnumber++ ) {
    switch( format ) {
    case RGB24 :
    case RGB15 :
    case RGB8 :
      sprintf( fname, "%s.%04d.ppm", argv[2], fnumber );
      ppm_write( fname, r, g, b, ysize, xsize );
      free( r );
      free( g );
      free( b );
      break;
    case BW8 :
      sprintf( fname, "%s.%04d.pgm", argv[2], fnumber );
      pgm_write( fname, r, ysize, xsize );
      free( r );
      break;
    default :
      fprintf( stderr, "Error: illegal format encoutered\n" );
      exit( YES );
    }
  }

  fprintf( stdout, "Converted %d %s to pnm files\n", fnumber,
	  fnumber == 1 ? "frame" : "frames" );
  fclose( fp );


}





/* ------------------------------------------------------------------
The following code is adapted from netpbm package by Jef Poskanzer.

	Copyright (C) 1988 by Jef Poskanzer.
--------------------------------------------------------------------- */


char mpiv_getc( FILE *file )
{
  int ich;
  char ch;

  ich = getc( file );
  if ( ich == EOF ) return (char) EOF;
  ch = (char) ich;
    
  if ( ch == '#' ) {
    do {
      ich = getc( file );
      if ( ich == EOF ) return (char) EOF;
      ch = (char) ich;
    } while ( ch != '\n' && ch != '\r' );
  }
  return ch;
}


int mpiv_getint( FILE *file )
{
  char ch;
  int i;

  do {
    ch = mpiv_getc( file );
  } while ( ch == ' ' || ch == '\t' || ch == '\n' || ch == '\r' );

  if ( ch < '0' || ch > '9' ) return -1;

  i = 0;
  do {
    i = i * 10 + ch - '0';
    ch = mpiv_getc( file );
  } while ( ch >= '0' && ch <= '9' );

  return i;
}




